=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2007-04-17 21:19:29 +0000
+++ bzrlib/annotate.py	2007-07-06 20:25:52 +0000
@@ -30,6 +30,7 @@
 
 from bzrlib import (
     errors,
+    osutils,
     patiencediff,
     tsort,
     )
@@ -79,7 +80,19 @@
             anno = "%-*s %-7s " % (max_revno_len, revno_str, author[:7])
 
         if anno.lstrip() == "" and full: anno = prevanno
-        print >>to_file, '%s| %s' % (anno, text)
+        try:
+            to_file.write(anno)
+        except UnicodeEncodeError:
+            # cmd_annotate should be passing in an 'exact' object, which means
+            # we have a direct handle to sys.stdout or equivalent. It may not
+            # be able to handle the exact Unicode characters, but 'annotate' is
+            # a user function (non-scripting), so shouldn't die because of
+            # unrepresentable annotation characters. So encode using 'replace',
+            # and write them again.
+            encoding = getattr(to_file, 'encoding', None) or \
+                    osutils.get_terminal_encoding()
+            to_file.write(anno.encode(encoding, 'replace'))
+        print >>to_file, '| %s' % (text,)
         prevanno=anno
 
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-07-04 08:08:08 +0000
+++ bzrlib/builtins.py	2007-07-06 17:21:57 +0000
@@ -3100,6 +3100,7 @@
                      'revision',
                      'show-ids',
                      ]
+    encoding_type = 'exact'
 
     @display_command
     def run(self, filename, all=False, long=False, revision=None,
@@ -3120,7 +3121,7 @@
                 raise errors.NotVersionedError(filename)
             tree = branch.repository.revision_tree(revision_id)
             file_version = tree.inventory[file_id].revision
-            annotate_file(branch, file_version, file_id, long, all, sys.stdout,
+            annotate_file(branch, file_version, file_id, long, all, self.outf,
                           show_ids=show_ids)
         finally:
             branch.unlock()

=== modified file 'bzrlib/tests/test_annotate.py'
--- bzrlib/tests/test_annotate.py	2007-01-29 21:11:28 +0000
+++ bzrlib/tests/test_annotate.py	2007-07-08 13:50:13 +0000
@@ -16,6 +16,7 @@
 
 """Whitebox tests for annotate functionality."""
 
+import codecs
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -301,6 +302,42 @@
                              'rev-1_1_1_1_1_1_1 | sixth\n',
                              sio.getvalue())
 
+    def test_annotate_unicode_author(self):
+        tree1 = self.make_branch_and_tree('tree1')
+
+        self.build_tree_contents([('tree1/a', 'adi\xc3\xb3s')])
+        tree1.add(['a'], ['a-id'])
+        tree1.commit('a', rev_id='rev-1',
+                     committer=u'Pepe P\xe9rez <pperez@ejemplo.com>',
+                     timestamp=1166046000.00, timezone=0)
+
+        self.build_tree_contents([('tree1/b', 'bye')])
+        tree1.add(['b'], ['b-id'])
+        tree1.commit('b', rev_id='rev-2',
+                     committer=u'p\xe9rez',
+                     timestamp=1166046000.00, timezone=0)
+
+        # this passes if no exception is raised
+        to_file = StringIO()
+        annotate.annotate_file(tree1.branch, 'rev-1', 'a-id', to_file=to_file)
+
+        sio = StringIO()
+        to_file = codecs.getwriter('ascii')(sio)
+        to_file.encoding = 'ascii' # codecs does not set it
+        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
+        self.assertEqualDiff('2   p?rez   | bye\n', sio.getvalue())
+
+        # test now with to_file.encoding = None
+        to_file = tests.StringIOWrapper()
+        to_file.encoding = None
+        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
+        self.assertContainsRe('2   p.rez   | bye\n', to_file.getvalue())
+
+        # and when it does not exist
+        to_file = StringIO()
+        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
+        self.assertContainsRe('2   p.rez   | bye\n', to_file.getvalue())
+
 
 class TestReannotate(tests.TestCase):
 

